@@ -13,7 +13,7 @@ use std::convert::TryInto;
 use std::mem::discriminant;
 
 // A map from generic parameter name to the type being used here.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Default)]
 struct GenericMap<'a> {
     map: HashMap<String, &'a Node>,
     // Because we captured this map at a previous time, we may need to look up
@@ -22,17 +22,6 @@ struct GenericMap<'a> {
     past_ctx: Option<&'a Context<'a>>,
 }
 
-// At the top level of a validation, we won't have any past_ctx
-// because there cannot be any generic parameter map.
-impl Default for GenericMap<'_> {
-    fn default() -> Self {
-        Self {
-            map: HashMap::default(),
-            past_ctx: None,
-        }
-    }
-}
-
 #[derive(Clone)]
 struct Context<'a> {
     lookup: &'a dyn LookupContext,
@@ -1056,6 +1045,7 @@ where
 fn validate_control(ctl: &Control, value: &Value, ctx: &Context) -> ValidateResult {
     match ctl {
         Control::Size(ctl_size) => validate_control_size(ctl_size, value, ctx),
+        Control::Regexp(re) => validate_control_regexp(re, value),
     }
 }
 
@@ -1096,6 +1086,23 @@ fn validate_control_size(ctl: &CtlOpSize, value: &Value, ctx: &Context) -> Valid
     })
 }
 
+/// Validate the control operator "regexp"
+///
+/// `regexp` applies a regular expression to a text string.
+///
+fn validate_control_regexp(re: &CtlOpRegexp, value: &Value) -> ValidateResult {
+    match value {
+        Value::Text(text) => {
+            if re.re.is_match(text) {
+                Ok(())
+            } else {
+                Err(mismatch("regex mismatch"))
+            }
+        }
+        _ => Err(mismatch("tstr")),
+    }
+}
+
 fn validate_size_uint(size: u64, value: &Value) -> ValidateResult {
     match value {
         Value::Integer(x) => {
